using System;
using System.Reflection;
using System.Data;
using System.Data.OleDb;
using System.Web;
using System.Web.UI;
using System.IO;
using System.Windows.Forms;
using mshtml;

[assembly: AssemblyTitle("")]
[assembly: AssemblyDescription("")]
[assembly: AssemblyConfiguration("")]
[assembly: AssemblyCompany("")]
[assembly: AssemblyProduct("")]
[assembly: AssemblyCopyright("")]
[assembly: AssemblyTrademark("")]
[assembly: AssemblyCulture("")]
[assembly: AssemblyVersion("1.0.0.0")]
[assembly: AssemblyKeyFile("")]

namespace wwwSharp.ClrHost.Examples
{
	public class DataAccessDOM: MarshalByRefObject
	{
		const int maxStringLength = 255;
	
		private string m_ConnectionString = "";
		private string m_SqlQuery = "";
		private bool m_StopQuery = false;
		
		public string ConnectionString
		{
			get {return m_ConnectionString;}
			set {m_ConnectionString = value;}
		}
		
		public string SqlQuery
		{
			get {return m_SqlQuery;}
			set {m_SqlQuery = value;}
		}
		
		public void ConvertQueryToHtmlTable(IHTMLTable table, IHTMLElement progress)
		{
			OleDbConnection conn = new OleDbConnection(m_ConnectionString);
			try
			{
				Cursor.Current = Cursors.WaitCursor;

			    string strProgress = "";
			    if (progress != null) strProgress = progress.innerHTML;

				conn.Open();
				OleDbCommand cmd = new OleDbCommand(m_SqlQuery, conn);
				OleDbDataReader reader = cmd.ExecuteReader();
				try
				{
					WriteData(reader, table, progress);
				}
				finally
				{				
					reader.Close();
					if (progress != null) progress.innerHTML = strProgress;
				}
			}
			finally
			{
				conn.Close();
				Cursor.Current = Cursors.Default;
			}
		}
		
		public void StopQuery()
		{
			m_StopQuery = true;
		}
		
		public void GetTableNamesForSelect(IHTMLSelectElement select)
		{
			OleDbConnection conn = new OleDbConnection(m_ConnectionString);
			try
			{
				Cursor.Current = Cursors.WaitCursor;
			
				conn.Open();
				DataTable tables = conn.GetOleDbSchemaTable(OleDbSchemaGuid.Tables,
					new object[] {null, null, null, "TABLE"});
				WriteTables(tables, select);
			}
			finally
			{
				conn.Close();
				Cursor.Current = Cursors.Default;
			}
		}

		private void WriteData(IDataReader reader, IHTMLTable table, IHTMLElement progress)
		{
			m_StopQuery = false;

			//Cleanup table contents
			((IHTMLElement)table).innerText = "";
			IHTMLDocument2 document = (IHTMLDocument2)((IHTMLElement)table).document;

			//Create THEAD and TBODY elements
			IHTMLElement head = document.createElement("THEAD");
			IHTMLElement body = document.createElement("TBODY");

			((IHTMLDOMNode)table).appendChild((IHTMLDOMNode)head);
			((IHTMLDOMNode)table).appendChild((IHTMLDOMNode)body);

			//Fill header
			IHTMLTableRow rowHead = (IHTMLTableRow)document.createElement("TR");
			((IHTMLDOMNode)head).appendChild((IHTMLDOMNode)rowHead);

			DataTable metadata = reader.GetSchemaTable();
			foreach(DataRow rowMetadata in metadata.Rows)
			{
				int colWidth = 10;
				Type colType = (Type)rowMetadata["DataType"];
				if (colType == typeof(string))
				{
					colWidth = (int)rowMetadata["ColumnSize"];
					if (colWidth < 6) colWidth = 10;
					if (colWidth > 50) colWidth = 50;
				}
				else if (colType == typeof(DateTime))
				{
					colWidth = 20;
				}

				IHTMLElement colHeader = document.createElement("TH");
				((IHTMLDOMNode)rowHead).appendChild((IHTMLDOMNode)colHeader);
				colHeader.style.width = (colWidth + 5).ToString() + "ex";
				colHeader.innerText = (string)rowMetadata["ColumnName"];
			}
			metadata	= null;
			rowHead		= null;
			head		= null;

			int counter = 0;

			//Fill body
			while (reader.Read())
			{
				IHTMLTableRow rowBody = (IHTMLTableRow)document.createElement("TR");
				((IHTMLDOMNode)body).appendChild((IHTMLDOMNode)rowBody);

				for (int fld = 0; fld < reader.FieldCount; fld++)
				{
				    IHTMLElement cell = (IHTMLElement)rowBody.insertCell(-1);

					if (reader.IsDBNull(fld))
					{
						cell.innerHTML = "<SPAN class=\"nullValue\">&lt;NULL&gt;</SPAN>";
					}
					else
					{
						TypeCode code = Type.GetTypeCode(reader.GetFieldType(fld));
						switch (code)
						{
							case TypeCode.Boolean:
							case TypeCode.Byte:
							case TypeCode.Char:
							case TypeCode.DateTime:
							case TypeCode.Decimal:
							case TypeCode.Double:
							case TypeCode.Int16:
							case TypeCode.Int32:
							case TypeCode.Int64:
							case TypeCode.SByte:
							case TypeCode.Single:
							case TypeCode.UInt16:
							case TypeCode.UInt32:
							case TypeCode.UInt64:
								cell.innerText = reader.GetValue(fld).ToString();
								break;
							case TypeCode.String:
								string strValue = reader.GetString(fld);
								//Truncate long strings in this example.
								if (strValue.Length > maxStringLength)
									strValue = strValue.Substring(0, maxStringLength) + " ...";
								cell.innerText = strValue;
								break;
							default:
								cell.innerHTML = "<SPAN class=\"blobValue\">&lt;BLOB&gt;</SPAN>";
								break;
						}
					}
				}

				counter++;
				if (progress != null && (counter % 10 == 0))
				{
					progress.innerText = counter.ToString() + " rows fetched.";
					Application.DoEvents();

					if (m_StopQuery) break;
				}
			}
		}
		
		protected static void WriteTables(DataTable schema, IHTMLSelectElement select)
		{
			((IHTMLElement)select).innerText = "";
			IHTMLDocument2 document = (IHTMLDocument2)((IHTMLElement)select).document;;

			DataView sortedSchema = new DataView(schema,
				"TABLE_TYPE = 'TABLE' OR TABLE_TYPE = 'VIEW'",
				"TABLE_NAME", DataViewRowState.CurrentRows);
			
			for (int i = 0; i < sortedSchema.Count; i++)
			{
				string tblName = (string)sortedSchema[i]["TABLE_NAME"];
				
				IHTMLOptionElement option = (IHTMLOptionElement)document.createElement("OPTION");
				option.text = tblName;
				option.value = tblName;
				
				select.add((IHTMLElement)option, -1);
			}
		}
	}
}
